#include <iostream>
#include <stack>
#include <list>
#include <queue>

using namespace std;

/**
Tokom pandemije virusa Covid19 cijene zlata su veoma nestabilne.
Napišite program koji za dati niz cijena zlata kroz N dana za svaki dan računa broj uzastopnih dana koji mu prethode u kojima je cijena zlata
bila manja.
Ulaz: U prvom redu ulaza dat je broj N, a u narednom niz on N prirodnih brojeva koji predstavljaju cijene zlata.
Izlaz: Niz od N brojeva, gdje i-ti broj predstavlja broj dana neposredno prije i-tog u kojima je cijena zlata bila manja od cijene u i-tom danu.
Napomena: Maksimalan broj poena se dobija za rješenje koje samo jednom prolazi datim nizom (složenost O(N)).

Primjer:

5 10 15 10 13

0 1 2 0 1
**/

void cijene_zlata(int* cijene, int n) {
    int* rez = new int[n];
    stack<pair<int,int>> s;
    for (int i = 0; i < n; i++) {
        int c = cijene[i];
        int cnt = 0;
        while (!s.empty() && s.top().first < c) {
            cnt += (s.top().second + 1);
            s.pop();
        }
        rez[i] = cnt;
        s.push(make_pair(c, cnt));
    }
    for (int i = 0; i < n; i++) {
        cout << rez[i] << " ";
    }
    cout << endl;

    delete [] rez;
}

/**
Data je jednostruko-ulančana lista prirodnih brojeva.
Napisati program koji za svaki čvor u listi pronalazi i
štampa vrijednost prvog sljedećeg čvora u listi koji ima strogo veću vrijednost.
Ukoliko nema čvora sa većom vrijednošću u nastavku liste, štampati 0.
Na primjer ako je data lista: 2→1→5, štampa se: 5, 5, 0. Ako je lista: 2→7→4→3→5, štampa se: 7, 0, 5, 5, 0.
**/
void prvi_veci(list<int> l) {
    int* rez = new int[l.size()];
    stack<pair<int,int>> s;

    int pos = 0;
    for (int v: l) {
        while (!s.empty() && s.top().first < v) {
            rez[s.top().second] = v;
            s.pop();
        }
        s.push(make_pair(v, pos));
        pos++;
    }
    while (!s.empty()) {
        rez[s.top().second] = 0;
        s.pop();
    }

    for (int i = 0; i < l.size(); i++) {
        cout << rez[i] << " ";
    }
    cout << endl;

    delete [] rez;
}

/**
Broj X se smatra otpornim na rotaciju ukoliko se nakon rotacije svake njegove
cifre za 180 stepeni dobije validan broj.
 Rotacijom cifara 0, 1 i 8 dobijaju se iste te cifre.
 Cifre 2 i 5 se rotacijom pretvaraju jedna u drugu.
 Cifre 6 i 9 se takođe rotacijom pretvaraju jedna u drugu.
 Rotacijom ostalih cifara ne dobijaju se validne cifre.
Napisati program koji učitava prirodan broj N i štampa sve prirodne brojeve između 1
i N (uključujući i njih) koji su otporni na rotaciju.
Vremenska složenost biće uzimana u obzir prilikom ocjenjivanja.
**/
void otporni(int n) {
    queue<int> q;

    q.push(1);
    q.push(2);
    q.push(5);
    q.push(6);
    q.push(8);
    q.push(9);

    while (true) {
        int br = q.front();
        q.pop();

        if (br > n)
            break;

        cout << br << endl;

        q.push(10*br + 0);
        q.push(10*br + 1);
        q.push(10*br + 2);
        q.push(10*br + 5);
        q.push(10*br + 6);
        q.push(10*br + 8);
        q.push(10*br + 9);
    }
}

/**
Data je matrica dimenzija NxN popunjena brojevima 0, 1 koja predstavlja platno. Polja
na kojima je upisan broj 0 predstavljaju čiste djelove platna. Polja na kojima je upisan
broj 1 predstavljaju mrlje od mastila. U svakoj sekundi mrlja od mastila se širi na osam
susjednih polja, ali se istovremeno i uklanja sa polja na kojima je bila. Napisati
program koji štampa izgled platna nakon T sekundi.
U prvom redu ulaza unose se brojevi N i T, a zatim se u sljedećin N redova unosi po
N elemenata matrice. Na konzoli ispisati matricu dimenzija NxN popunjenu brojevima
0, 1 koja predstavljaja izgled platna nakon T sekundi.
**/
void stampaj_matricu(int** mat, int n) {
    for (int i = 0; i < n; i++) {
        for (int j = 0; j < n; j++) {
            cout << mat[i][j] << " ";
        }
        cout << endl;
    }
}

void platno(int** mat, int n, int T) {
    queue<pair<pair<int,int>,int>> q;

    for (int i = 0; i < n; i++) {
        for (int j = 0; j < n; j++) {
            if (mat[i][j] == 1)
                q.push(make_pair(make_pair(i, j), 0));
        }
    }

    int di[] = {-1, -1, -1, 0, 0, 1, 1, 1};
    int dj[] = {-1, 0, 1, -1, 1, -1, 0, 1};

    while (true) {
        int i = q.front().first.first;
        int j = q.front().first.second;
        int t = q.front().second;
        q.pop();

        if (t == T)
            break;

        for (int k = 0; k < 8; k++) {
            int newi = i + di[k];
            int newj = j + dj[k];

            if (newi >= 0 && newi < n && newj >= 0 && newj < n) {
                mat[newi][newj] = 1;
                q.push(make_pair(make_pair(newi, newj), t+1));
            }
        }

        mat[i][j] = 0;
    }
}

int** unesi_matricu(int n) {
    int** mat = new int*[n];
    for(int i=0; i<n; i++){
        mat[i] = new int[n];
    }

    for(int i=0; i<n;i++){
        for(int j=0; j<n; j++){
            cin >> mat[i][j];
        }
    }

    return mat;
}

void oslobodi_matricu(int** mat, int n) {
    for (int i = 0; i < n; i++) {
        delete [] mat[i];
    }
    delete [] mat;
}

int main()
{
    int cijene[] = {5, 10, 15, 10, 13};
    cijene_zlata(cijene, 5);

    list<int> l;
    l.push_back(2);
    l.push_back(7);
    l.push_back(4);
    l.push_back(3);
    l.push_back(5);

    prvi_veci(l);

    otporni(153);

    int n, t;
    cin >> n >> t;

    int** mat = unesi_matricu(n);
    platno(mat, n, t);
    stampaj_matricu(mat, n);
    oslobodi_matricu(mat, n);

    return 0;
}
